<?php
/**
 * Plugin Messages (addon for plugin Adm_redacteurs)
 * Version  : 1.1 (2017/10/23)
 * Compatibility : Guppy v5.0+ et Adm_redacteurs v6.8+
 * Author   : jérôme CROUX (jchouix)
 * Web site : http://lebrikabrak.info/
 * E-mail   : lebrikabrak@free.fr
 * Licence  : GNU Lesser General Public License
 */

defined('ADMR_DIRNAME') OR exit('No direct script access allowed');

class Messages_model extends Model
{    
    /*
    |--------------------------------------------------------------------------
    | Properties
    |--------------------------------------------------------------------------
    */
    private $_data_DB = array(); // Données de la base de données
    private $_file_dir_path = MESSAGES_CONTENTS_PATH; // Chemin du répertoire contenant les fichiers des données des messages
    private $_file_prefix = MESSAGES_CONTENTS_FILE_PREFIX; // Préfixe du nom du fichier contenant les données du message
    private $_file_ext = INCEXT;

    private $_params = array(
        'id' => 0, // Id du document
        'title' => '', // Titre du document
        'contents' => '', // contenu du document
        'author_id' => 0, // Id de l'auteur du document (0 = admin)
        'receiver_id' => 0, // Id du ou des destinataires du document (0 = admin)
        'create_datetime' => '', // Date de création (timestamp)
    );

    /*
    |--------------------------------------------------------------------------
    | Constructor
    |--------------------------------------------------------------------------
    */
    public function __construct()
	{

	}

	// --------------------------------------------------------------------

	/**
	 * Renvoie le nom des fichiers enregistrés dans la base de donnée
     * 
     * @access public 
	 * @return boolean
	 */
    public function get_all_files()
	{
        $files = array();
        
        if (!empty($this -> _file_dir_path)) {
            $files = array_slice(scandir($this -> _file_dir_path), 2);
            unset($files[array_search('index.php', $files)]); // Suppression du fichier index.php
        }
        
        return $files;
	}

	// --------------------------------------------------------------------

	/**
	 * Renvoie les identifiants des fichiers enregistrés dans la base de donnée
     * 
     * @access public 
	 * @return boolean
	 */
    public function get_files_ids()
	{
        $files_ids = array();
        $files = $this -> get_all_files();

        foreach($files as $key => $file) {
            $files_ids[] = str_replace(array($this -> _file_prefix, $this -> _file_ext), '', $file);
        }
        
        return $files_ids;
	}

	// --------------------------------------------------------------------

	/**
	 * Vérifie si le document existe ou non
     * 
     * @access public 
	 * @param integer Identifiant du document
	 * @return boolean
	 */
    public function file_exists($doc_id = 0)
	{
        if (!empty($doc_id) && ctype_digit((string) $doc_id) && is_file($this -> _get_file_path($doc_id))) {            
            return TRUE;    
        }
        
        return FALSE;
	}

	// --------------------------------------------------------------------

	/**
	 * Lit et renvoie dans un tableau les données enregistrées dans le document identifié par la valeur de son id
     * 
     * @access public 
	 * @param integer Identifiant du document
	 * @return array Tableau contenant les valeurs du document
	 */
    public function read($doc_id = 0)
    {
        $data = array_merge($this -> _params,
            array( // Données par défaut
            )
        );
        
        if ($this -> file_exists($doc_id)) {
            $data = array_merge($data, (array) unserialize(file_get_contents($this -> _get_file_path($doc_id))));    
        }
       
        return $data; 
    }
    
	// --------------------------------------------------------------------
 
	/**
	 * Ecrit les données du document dans un fichier
     * 
     * @access public 
     * @param array Données du rédacteur ou du groupe rédacteur
	 * @return array
	 */    
    public function write($data)
    {
        settype($data, 'array');

        if (empty($data['id']) || ! ctype_digit((string) $data['id'])) {
            return FALSE;
        }
      
        $file_path = $this -> _get_file_path($data['id']);
        $data = $this -> _clean($data);
                
		return file_put_contents($file_path, serialize($data), LOCK_EX);
    }

	// --------------------------------------------------------------------
 
	/**
	 * Supprime les documents identifiés par leur id
     * 
     * @access public 
     * @param array|integer Identifiants des documents à supprimer
	 * @return boolean
	 */    
    public function delete_by_id($doc_ids)
    {
        settype($doc_ids, 'array');

        if (empty($this -> _file_dir_path) || empty($doc_ids)) {
            return FALSE;
        }
                
        foreach($doc_ids as $doc_id) {
            if ($this -> file_exists($doc_id)) {
                @unlink($this -> _get_file_path($doc_id));    
            }
        }
       
        return TRUE; 
    }
   
	// --------------------------------------------------------------------
 
	/**
	 * Retourne le chemin du fichier associé à l'id passé en paramètre
     * 
     * @access private 
     * @param integer Id du document
	 * @return string Chemin du fichier associé à l'id passé en paramètre
	 */    
    private function _get_file_path($doc_id)
    {
        return $this -> _file_dir_path.'/'.$this -> _file_prefix.$doc_id.$this -> _file_ext;    
    }

	// --------------------------------------------------------------------
 
	/**
	 * Nettoie les données du rédacteur
     * 
     * @access private 
	 * @param array Données du rédacteur à nettoyer
	 * @return array
	 */        
    private function _clean($data)
    {
        $data_by_default = $this -> _params;
        $data = $this -> _parse_params_contents($data);
                
        return array_merge($data_by_default, array_intersect_key($data, $data_by_default));
    }

	// --------------------------------------------------------------------
 
	/**
	 * Nettoie les données de type "contenu"
     * 
     * @access private 
	 * @param array Données à nettoyer
	 * @return array
	 */
    private function _parse_params_contents($params)
    {
        $in = array(
    		'\\',
    		'="'.ADMR_SITE_URL
    	);

    	$out = array(
    		'&#92;',
    		'="'
    	);       

        $params['contents'] = str_replace($in, $out, $params['contents']);
        
        return $params;
    }

	// --------------------------------------------------------------------
  
	/**
	 * Définit et renvoie les règles de validation des paramètres
     * 
     * @access public 
	 * @param array Tableau contenant les paramètres à valider
	 * @param array Tableau contenant les messages d'erreur
	 * @param array Tableau contenant les options de configuration
	 * @return array Tableau contenant les règles de validation et les messages d'erreur associés
	 */
    public function set_params_rules($params, $lang, $config)
    {
        return array_merge(
            Params_rules_helper :: set_params_title_rules($lang, $config),    // Titre du document
            array(
   /*             'nb_questions' => array( // Nombre de questions du filehtml
                    array(
                        'rules' => array(array('Var_helper', 'is_between[1,'.$config['filehtml']['nb_questions_max'].']')),
                        'messages' => htmlspecialchars($lang['form_errors_messages']['field_not_valid'], ENT_QUOTES, 'UTF-8')
                    ),
                ),*/
            )
        );
    }
     
	// --------------------------------------------------------------------

} // End of Class